# coding: utf8
from __future__ import unicode_literals

import plac
import json
import spacy
import numpy
from pathlib import Path

from ..vectors import Vectors
from ..util import prints, ensure_path


@plac.annotations(
    lang=("model language", "positional", None, str),
    output_dir=("model output directory", "positional", None, Path),
    lexemes_loc=("location of JSONL-formatted lexical data", "positional",
                 None, Path),
    vectors_loc=("optional: location of vectors data, as numpy .npz",
                 "positional", None, str),
    prune_vectors=("optional: number of vectors to prune to.",
                   "option", "V", int)
)
def make_vocab(lang, output_dir, lexemes_loc, vectors_loc=None, prune_vectors=-1):
    """Compile a vocabulary from a lexicon jsonl file and word vectors."""
    if not lexemes_loc.exists():
        prints(lexemes_loc, title="Can't find lexical data", exits=1)
    vectors_loc = ensure_path(vectors_loc)
    nlp = spacy.blank(lang)
    for word in nlp.vocab:
        word.rank = 0
    lex_added = 0
    with lexemes_loc.open() as file_:
        for line in file_:
            if line.strip():
                attrs = json.loads(line)
                if 'settings' in attrs:
                    nlp.vocab.cfg.update(attrs['settings'])
                else:
                    lex = nlp.vocab[attrs['orth']]
                    lex.set_attrs(**attrs)
                    assert lex.rank == attrs['id']
                lex_added += 1
    if vectors_loc is not None:
        vector_data = numpy.load(vectors_loc.open('rb'))
        nlp.vocab.vectors = Vectors(data=vector_data)
        for word in nlp.vocab:
            if word.rank:
                nlp.vocab.vectors.add(word.orth, row=word.rank)

    if prune_vectors >= 1:
        remap = nlp.vocab.prune_vectors(prune_vectors)
    if not output_dir.exists():
        output_dir.mkdir()
    nlp.to_disk(output_dir)
    vec_added = len(nlp.vocab.vectors)
    prints("{} entries, {} vectors".format(lex_added, vec_added), output_dir,
           title="Sucessfully compiled vocab and vectors, and saved model")
    return nlp



# coding: utf8
from __future__ import unicode_literals

from ...symbols import ORTH, LEMMA, TAG, NORM, PRON_LEMMA
import re

_exc = {}

# time
for exc_data in [
    {LEMMA: "قبل الميلاد", ORTH: "ق.م"},
    {LEMMA: "بعد الميلاد", ORTH: "ب. م"},
    {LEMMA: "ميلادي", ORTH: ".م"},
    {LEMMA: "هجري", ORTH: ".هـ"},
    {LEMMA: "توفي", ORTH: ".ت"}]:
    _exc[exc_data[ORTH]] = [exc_data]

# scientific abv.
for exc_data in [
    {LEMMA: "صلى الله عليه وسلم", ORTH: "صلعم"},
    {LEMMA: "الشارح", ORTH: "الشـ"},
    {LEMMA: "الظاهر", ORTH: "الظـ"},
    {LEMMA: "أيضًا", ORTH: "أيضـ"},
    {LEMMA: "إلى آخره", ORTH: "إلخ"},
    {LEMMA: "انتهى", ORTH: "اهـ"},
    {LEMMA: "حدّثنا", ORTH: "ثنا"},
    {LEMMA: "حدثني", ORTH: "ثنى"},
    {LEMMA: "أنبأنا", ORTH: "أنا"},
    {LEMMA: "أخبرنا", ORTH: "نا"},
    {LEMMA: "مصدر سابق", ORTH: "م. س"},
    {LEMMA: "مصدر نفسه", ORTH: "م. ن"}]:
    _exc[exc_data[ORTH]] = [exc_data]

# other abv.
for exc_data in [
    {LEMMA: "دكتور", ORTH: "د."},
    {LEMMA: "أستاذ دكتور", ORTH: "أ.د"},
    {LEMMA: "أستاذ", ORTH: "أ."},
    {LEMMA: "بروفيسور", ORTH: "ب."}]:
    _exc[exc_data[ORTH]] = [exc_data]

for exc_data in [
    {LEMMA: "تلفون", ORTH: "ت."},
    {LEMMA: "صندوق بريد", ORTH: "ص.ب"}]:
    _exc[exc_data[ORTH]] = [exc_data]

TOKENIZER_EXCEPTIONS = _exc



# coding: utf8
from __future__ import unicode_literals

from .lex_attrs import LEX_ATTRS
from .punctuation import TOKENIZER_INFIXES
from .stop_words import STOP_WORDS
from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS
from ..tokenizer_exceptions import BASE_EXCEPTIONS
from ...attrs import LANG
from ...language import Language
from ...util import update_exc


class TatarDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters[LANG] = lambda text: 'tt'

    lex_attr_getters.update(LEX_ATTRS)

    tokenizer_exceptions = update_exc(BASE_EXCEPTIONS, TOKENIZER_EXCEPTIONS)
    infixes = tuple(TOKENIZER_INFIXES)

    stop_words = STOP_WORDS


class Tatar(Language):
    lang = 'tt'
    Defaults = TatarDefaults


__all__ = ['Tatar']



# coding: utf8
from __future__ import unicode_literals

from ...symbols import ORTH, NORM

_exc = {
    "sağol": [
        {ORTH: "sağ"},
        {ORTH: "ol", NORM: "olun"}]
}


for exc_data in [
    {ORTH: "A.B.D.", NORM: "Amerika Birleşik Devletleri"},
    {ORTH: "Alb.", NORM: "Albay"},
    {ORTH: "Ar.Gör.", NORM: "Araştırma Görevlisi"},
    {ORTH: "Arş.Gör.", NORM: "Araştırma Görevlisi"},
    {ORTH: "Asb.", NORM: "Astsubay"},
    {ORTH: "Astsb.", NORM: "Astsubay"},
    {ORTH: "As.İz.", NORM: "Askeri İnzibat"},
    {ORTH: "Atğm", NORM: "Asteğmen"},
    {ORTH: "Av.", NORM: "Avukat"},
    {ORTH: "Apt.", NORM: "Apartmanı"},
    {ORTH: "Bçvş.", NORM: "Başçavuş"},
    {ORTH: "bk.", NORM: "bakınız"},
    {ORTH: "bknz.", NORM: "bakınız"},
    {ORTH: "Bnb.", NORM: "Binbaşı"},
    {ORTH: "bnb.", NORM: "binbaşı"},
    {ORTH: "Böl.", NORM: "Bölümü"},
    {ORTH: "Bşk.", NORM: "Başkanlığı"},
    {ORTH: "Bştbp.", NORM: "Baştabip"},
    {ORTH: "Bul.", NORM: "Bulvarı"},
    {ORTH: "Cad.", NORM: "Caddesi"},
    {ORTH: "çev.", NORM: "çeviren"},
    {ORTH: "Çvş.", NORM: "Çavuş"},
    {ORTH: "dak.", NORM: "dakika"},
    {ORTH: "dk.", NORM: "dakika"},
    {ORTH: "Doç.", NORM: "Doçent"},
    {ORTH: "doğ.", NORM: "doğum tarihi"},
    {ORTH: "drl.", NORM: "derleyen"},
    {ORTH: "Dz.", NORM: "Deniz"},
    {ORTH: "Dz.K.K.lığı", NORM: "Deniz Kuvvetleri Komutanlığı"},
    {ORTH: "Dz.Kuv.", NORM: "Deniz Kuvvetleri"},
    {ORTH: "Dz.Kuv.K.", NORM: "Deniz Kuvvetleri Komutanlığı"},
    {ORTH: "dzl.", NORM: "düzenleyen"},
    {ORTH: "Ecz.", NORM: "Eczanesi"},
    {ORTH: "ekon.", NORM: "ekonomi"},
    {ORTH: "Fak.", NORM: "Fakültesi"},
    {ORTH: "Gn.", NORM: "Genel"},
    {ORTH: "Gnkur.", NORM: "Genelkurmay"},
    {ORTH: "Gn.Kur.", NORM: "Genelkurmay"},
    {ORTH: "gr.", NORM: "gram"},
    {ORTH: "Hst.", NORM: "Hastanesi"},
    {ORTH: "Hs.Uzm.", NORM: "Hesap Uzmanı"},
    {ORTH: "huk.", NORM: "hukuk"},
    {ORTH: "Hv.", NORM: "Hava"},
    {ORTH: "Hv.K.K.lığı", NORM: "Hava Kuvvetleri Komutanlığı"},
    {ORTH: "Hv.Kuv.", NORM: "Hava Kuvvetleri"},
    {ORTH: "Hv.Kuv.K.", NORM: "Hava Kuvvetleri Komutanlığı"},
    {ORTH: "Hz.", NORM: "Hazreti"},
    {ORTH: "Hz.Öz.", NORM: "Hizmete Özel"},
    {ORTH: "İng.", NORM: "İngilizce"},
    {ORTH: "Jeol.", NORM: "Jeoloji"},
    {ORTH: "jeol.", NORM: "jeoloji"},
    {ORTH: "Korg.", NORM: "Korgeneral"},
    {ORTH: "Kur.", NORM: "Kurmay"},
    {ORTH: "Kur.Bşk.", NORM: "Kurmay Başkanı"},
    {ORTH: "Kuv.", NORM: "Kuvvetleri"},
    {ORTH: "Ltd.", NORM: "Limited"},
    {ORTH: "Mah.", NORM: "Mahallesi"},
    {ORTH: "mah.", NORM: "mahallesi"},
    {ORTH: "max.", NORM: "maksimum"},
    {ORTH: "min.", NORM: "minimum"},
    {ORTH: "Müh.", NORM: "Mühendisliği"},
    {ORTH: "müh.", NORM: "mühendisliği"},
    {ORTH: "MÖ.", NORM: "Milattan Önce"},
    {ORTH: "Onb.", NORM: "Onbaşı"},
    {ORTH: "Ord.", NORM: "Ordinaryüs"},
    {ORTH: "Org.", NORM: "Orgeneral"},
    {ORTH: "Ped.", NORM: "Pedagoji"},
    {ORTH: "Prof.", NORM: "Profesör"},
    {ORTH: "Sb.", NORM: "Subay"},
    {ORTH: "Sn.", NORM: "Sayın"},
    {ORTH: "sn.", NORM: "saniye"},
    {ORTH: "Sok.", NORM: "Sokak"},
    {ORTH: "Şb.", NORM: "Şube"},
    {ORTH: "Şti.", NORM: "Şirketi"},
    {ORTH: "Tbp.", NORM: "Tabip"},
    {ORTH: "T.C.", NORM: "Türkiye Cumhuriyeti"},
    {ORTH: "Tel.", NORM: "Telefon"},
    {ORTH: "tel.", NORM: "telefon"},
    {ORTH: "telg.", NORM: "telgraf"},
    {ORTH: "Tğm.", NORM: "Teğmen"},
    {ORTH: "tğm.", NORM: "teğmen"},
    {ORTH: "tic.", NORM: "ticaret"},
    {ORTH: "Tug.", NORM: "Tugay"},
    {ORTH: "Tuğg.", NORM: "Tuğgeneral"},
    {ORTH: "Tümg.", NORM: "Tümgeneral"},
    {ORTH: "Uzm.", NORM: "Uzman"},
    {ORTH: "Üçvş.", NORM: "Üstçavuş"},
    {ORTH: "Üni.", NORM: "Üniversitesi"},
    {ORTH: "Ütğm.", NORM: "Üsteğmen"},
    {ORTH: "vb.", NORM: "ve benzeri"},
    {ORTH: "vs.", NORM: "vesaire"},
    {ORTH: "Yard.", NORM: "Yardımcı"},
    {ORTH: "Yar.", NORM: "Yardımcı"},
    {ORTH: "Yd.Sb.", NORM: "Yedek Subay"},
    {ORTH: "Yard.Doç.", NORM: "Yardımcı Doçent"},
    {ORTH: "Yar.Doç.", NORM: "Yardımcı Doçent"},
    {ORTH: "Yb.", NORM: "Yarbay"},
    {ORTH: "Yrd.", NORM: "Yardımcı"},
    {ORTH: "Yrd.Doç.", NORM: "Yardımcı Doçent"},
    {ORTH: "Y.Müh.", NORM: "Yüksek mühendis"},
    {ORTH: "Y.Mim.", NORM: "Yüksek mimar"}]:
    _exc[exc_data[ORTH]] = [exc_data]


for orth in [
    "Dr.", "yy."]:
    _exc[orth] = [{ORTH: orth}]


TOKENIZER_EXCEPTIONS = _exc



# cython: infer_types=True
# coding: utf8
from __future__ import unicode_literals

from libc.string cimport memcpy
from cpython.mem cimport PyMem_Malloc, PyMem_Free
# Compiler crashes on memory view coercion without this. Should report bug.
from cython.view cimport array as cvarray
cimport numpy as np
np.import_array()
import numpy

from ..typedefs cimport hash_t
from ..lexeme cimport Lexeme
from .. import parts_of_speech
from ..attrs cimport IS_ALPHA, IS_ASCII, IS_DIGIT, IS_LOWER, IS_PUNCT, IS_SPACE
from ..attrs cimport IS_BRACKET, IS_QUOTE, IS_LEFT_PUNCT, IS_RIGHT_PUNCT
from ..attrs cimport IS_OOV, IS_TITLE, IS_UPPER, IS_CURRENCY, LIKE_URL, LIKE_NUM, LIKE_EMAIL
from ..attrs cimport IS_STOP, ID, ORTH, NORM, LOWER, SHAPE, PREFIX, SUFFIX
from ..attrs cimport LENGTH, CLUSTER, LEMMA, POS, TAG, DEP
from ..compat import is_config
from ..errors import Errors
from .. import util
from .underscore import Underscore, get_ext_args


cdef class Token:
    """An individual token – i.e. a word, punctuation symbol, whitespace,
    etc."""
    @classmethod
    def set_extension(cls, name, **kwargs):
        if cls.has_extension(name) and not kwargs.get('force', False):
            raise ValueError(Errors.E090.format(name=name, obj='Token'))
        Underscore.token_extensions[name] = get_ext_args(**kwargs)

    @classmethod
    def get_extension(cls, name):
        return Underscore.token_extensions.get(name)

    @classmethod
    def has_extension(cls, name):
        return name in Underscore.token_extensions

    @classmethod
    def remove_extension(cls, name):
        if not cls.has_extension(name):
            raise ValueError(Errors.E046.format(name=name))
        return Underscore.token_extensions.pop(name)

    def __cinit__(self, Vocab vocab, Doc doc, int offset):
        """Construct a `Token` object.

        vocab (Vocab): A storage container for lexical types.
        doc (Doc): The parent document.
        offset (int): The index of the token within the document.
        """
        self.vocab = vocab
        self.doc = doc
        self.c = &self.doc.c[offset]
        self.i = offset

    def __hash__(self):
        return hash((self.doc, self.i))

    def __len__(self):
        """The number of unicode characters in the token, i.e. `token.text`.

        RETURNS (int): The number of unicode characters in the token.
        """
        return self.c.lex.length

    def __unicode__(self):
        return self.text

    def __bytes__(self):
        return self.text.encode('utf8')

    def __str__(self):
        if is_config(python3=True):
            return self.__unicode__()
        return self.__bytes__()

    def __repr__(self):
        return self.__str__()

    def __richcmp__(self, Token other, int op):
        # http://cython.readthedocs.io/en/latest/src/userguide/special_methods.html
        if other is None:
            if op in (0, 1, 2):
                return False
            else:
                return True
        cdef Doc my_doc = self.doc
        cdef Doc other_doc = other.doc
        my = self.idx
        their = other.idx
        if op == 0:
            return my < their
        elif op == 2:
            if my_doc is other_doc:
                return my == their
            else:
                return False
        elif op == 4:
            return my > their
        elif op == 1:
            return my <= their
        elif op == 3:
            if my_doc is other_doc:
                return my != their
            else:
                return True
        elif op == 5:
            return my >= their
        else:
            raise ValueError(Errors.E041.format(op=op))

    @property
    def _(self):
        return Underscore(Underscore.token_extensions, self,
                          start=self.idx, end=None)

    cpdef bint check_flag(self, attr_id_t flag_id) except -1:
        """Check the value of a boolean flag.

        flag_id (int): The ID of the flag attribute.
        RETURNS (bool): Whether the flag is set.

        EXAMPLE:
            >>> from spacy.attrs import IS_TITLE
            >>> doc = nlp(u'Give it back! He pleaded.')
            >>> token = doc[0]
            >>> token.check_flag(IS_TITLE)
            True
        """
        return Lexeme.c_check_flag(self.c.lex, flag_id)

    def nbor(self, int i=1):
        """Get a neighboring token.

        i (int): The relative position of the token to get. Defaults to 1.
        RETURNS (Token): The token at position `self.doc[self.i+i]`.
        """
        if self.i+i < 0 or (self.i+i >= len(self.doc)):
            raise IndexError(Errors.E042.format(i=self.i, j=i, length=len(self.doc)))
        return self.doc[self.i+i]

    def similarity(self, other):
        """Make a semantic similarity estimate. The default estimate is cosine
        similarity using an average of word vectors.

        other (object): The object to compare with. By default, accepts `Doc`,
            `Span`, `Token` and `Lexeme` objects.
        RETURNS (float): A scalar similarity score. Higher is more similar.
        """
        if 'similarity' in self.doc.user_token_hooks:
            return self.doc.user_token_hooks['similarity'](self)
        if hasattr(other, '__len__') and len(other) == 1 and hasattr(other, "__getitem__"):
            if self.c.lex.orth == getattr(other[0], 'orth', None):
                return 1.0
        elif hasattr(other, 'orth'):
            if self.c.lex.orth == other.orth:
                return 1.0
        self_norm = self.vector_norm
        other_norm = other.vector_norm
        if self_norm == 0 or other_norm == 0:
            return 0.0
        return (numpy.dot(self.vector, other.vector) /
                (self_norm * other_norm))

    property lex_id:
        """RETURNS (int): Sequential ID of the token's lexical type."""
        def __get__(self):
            return self.c.lex.id

    property rank:
        """RETURNS (int): Sequential ID of the token's lexical type, used to
        index into tables, e.g. for word vectors."""
        def __get__(self):
            return self.c.lex.id

    property string:
        """Deprecated: Use Token.text_with_ws instead."""
        def __get__(self):
            return self.text_with_ws

    property text:
        """RETURNS (unicode): The original verbatim text of the token."""
        def __get__(self):
            return self.orth_

    property text_with_ws:
        """RETURNS (unicode): The text content of the span (with trailing
            whitespace).
        """
        def __get__(self):
            cdef unicode orth = self.vocab.strings[self.c.lex.orth]
            if self.c.spacy:
                return orth + u' '
            else:
                return orth

    property prob:
        """RETURNS (float): Smoothed log probability estimate of token type."""
        def __get__(self):
            return self.c.lex.prob

    property sentiment:
        """RETURNS (float): A scalar value indicating the positivity or
            negativity of the token."""
        def __get__(self):
            if 'sentiment' in self.doc.user_token_hooks:
                return self.doc.user_token_hooks['sentiment'](self)
            return self.c.lex.sentiment

    property lang:
        """RETURNS (uint64): ID of the language of the parent document's
            vocabulary.
        """
        def __get__(self):
            return self.c.lex.lang

    property idx:
        """RETURNS (int): The character offset of the token within the parent
            document.
        """
        def __get__(self):
            return self.c.idx

    property cluster:
        """RETURNS (int): Brown cluster ID."""
        def __get__(self):
            return self.c.lex.cluster

    property orth:
        """RETURNS (uint64): ID of the verbatim text content."""
        def __get__(self):
            return self.c.lex.orth

    property lower:
        """RETURNS (uint64): ID of the lowercase token text."""
        def __get__(self):
            return self.c.lex.lower

    property norm:
        """RETURNS (uint64): ID of the token's norm, i.e. a normalised form of
            the token text. Usually set in the language's tokenizer exceptions
            or norm exceptions.
        """
        def __get__(self):
            return self.c.lex.norm

    property shape:
        """RETURNS (uint64): ID of the token's shape, a transform of the
            tokens's string, to show orthographic features (e.g. "Xxxx", "dd").
        """
        def __get__(self):
            return self.c.lex.shape

    property prefix:
        """RETURNS (uint64): ID of a length-N substring from the start of the
            token. Defaults to `N=1`.
        """
        def __get__(self):
            return self.c.lex.prefix

    property suffix:
        """RETURNS (uint64): ID of a length-N substring from the end of the
            token. Defaults to `N=3`.
        """
        def __get__(self):
            return self.c.lex.suffix

    property lemma:
        """RETURNS (uint64): ID of the base form of the word, with no
            inflectional suffixes.
        """
        def __get__(self):
            if self.c.lemma == 0:
                lemma_ = self.vocab.morphology.lemmatizer.lookup(self.orth_)
                return self.vocab.strings[lemma_]
            else:
                return self.c.lemma

        def __set__(self, attr_t lemma):
            self.c.lemma = lemma

    property pos:
        """RETURNS (uint64): ID of coarse-grained part-of-speech tag."""
        def __get__(self):
            return self.c.pos

    property tag:
        """RETURNS (uint64): ID of fine-grained part-of-speech tag."""
        def __get__(self):
            return self.c.tag

        def __set__(self, attr_t tag):
            self.vocab.morphology.assign_tag(self.c, tag)

    property dep:
        """RETURNS (uint64): ID of syntactic dependency label."""
        def __get__(self):
            return self.c.dep

        def __set__(self, attr_t label):
            self.c.dep = label

    property has_vector:
        """A boolean value indicating whether a word vector is associated with
        the object.

        RETURNS (bool): Whether a word vector is associated with the object.
        """
        def __get__(self):
            if 'has_vector' in self.doc.user_token_hooks:
                return self.doc.user_token_hooks['has_vector'](self)
            if self.vocab.vectors.size == 0 and self.doc.tensor.size != 0:
                return True
            return self.vocab.has_vector(self.c.lex.orth)

    property vector:
        """A real-valued meaning representation.

        RETURNS (numpy.ndarray[ndim=1, dtype='float32']): A 1D numpy array
            representing the token's semantics.
        """
        def __get__(self):
            if 'vector' in self.doc.user_token_hooks:
                return self.doc.user_token_hooks['vector'](self)
            if self.vocab.vectors.size == 0 and self.doc.tensor.size != 0:
                return self.doc.tensor[self.i]
            else:
                return self.vocab.get_vector(self.c.lex.orth)

    property vector_norm:
        """The L2 norm of the token's vector representation.

        RETURNS (float): The L2 norm of the vector representation.
        """
        def __get__(self):
            if 'vector_norm' in self.doc.user_token_hooks:
                return self.doc.user_token_hooks['vector_norm'](self)
            vector = self.vector
            return numpy.sqrt((vector ** 2).sum())

    property n_lefts:
        """RETURNS (int): The number of leftward immediate children of the
            word, in the syntactic dependency parse.
        """
        def __get__(self):
            return self.c.l_kids

    property n_rights:
        """RETURNS (int): The number of rightward immediate children of the
            word, in the syntactic dependency parse.
        """
        def __get__(self):
            return self.c.r_kids

    property sent:
        """RETURNS (Span): The sentence span that the token is a part of."""
        def __get__(self):
            if 'sent' in self.doc.user_token_hooks:
                return self.doc.user_token_hooks['sent'](self)
            return self.doc[self.i : self.i+1].sent

    property sent_start:
        def __get__(self):
            # Raising a deprecation warning here causes errors for autocomplete
            # Handle broken backwards compatibility case: doc[0].sent_start
            # was False.
            if self.i == 0:
                return False
            else:
                return self.c.sent_start

        def __set__(self, value):
            self.is_sent_start = value

    property is_sent_start:
        """RETURNS (bool / None): Whether the token starts a sentence.
            None if unknown.
        """
        def __get__(self):
            if self.c.sent_start == 0:
                return None
            elif self.c.sent_start < 0:
                return False
            else:
                return True

        def __set__(self, value):
            if self.doc.is_parsed:
                raise ValueError(Errors.E043)
            if value is None:
                self.c.sent_start = 0
            elif value is True:
                self.c.sent_start = 1
            elif value is False:
                self.c.sent_start = -1
            else:
                raise ValueError(Errors.E044.format(value=value))

    property lefts:
        """The leftward immediate children of the word, in the syntactic
        dependency parse.

        YIELDS (Token): A left-child of the token.
        """
        def __get__(self):
            cdef int nr_iter = 0
            cdef const TokenC* ptr = self.c - (self.i - self.c.l_edge)
            while ptr < self.c:
                if ptr + ptr.head == self.c:
                    yield self.doc[ptr - (self.c - self.i)]
                ptr += 1
                nr_iter += 1
                # This is ugly, but it's a way to guard out infinite loops
                if nr_iter >= 10000000:
                    raise RuntimeError(Errors.E045.format(attr='token.lefts'))

    property rights:
        """The rightward immediate children of the word, in the syntactic
        dependency parse.

        YIELDS (Token): A right-child of the token.
        """
        def __get__(self):
            cdef const TokenC* ptr = self.c + (self.c.r_edge - self.i)
            tokens = []
            cdef int nr_iter = 0
            while ptr > self.c:
                if ptr + ptr.head == self.c:
                    tokens.append(self.doc[ptr - (self.c - self.i)])
                ptr -= 1
                nr_iter += 1
                if nr_iter >= 10000000:
                    raise RuntimeError(Errors.E045.format(attr='token.rights'))
            tokens.reverse()
            for t in tokens:
                yield t

    property children:
        """A sequence of the token's immediate syntactic children.

        YIELDS (Token): A child token such that child.head==self
        """
        def __get__(self):
            yield from self.lefts
            yield from self.rights

    property subtree:
        """A sequence of all the token's syntactic descendents.

        YIELDS (Token): A descendent token such that
            `self.is_ancestor(descendent)`.
        """
        def __get__(self):
            for word in self.lefts:
                yield from word.subtree
            yield self
            for word in self.rights:
                yield from word.subtree

    property left_edge:
        """The leftmost token of this token's syntactic descendents.

        RETURNS (Token): The first token such that `self.is_ancestor(token)`.
        """
        def __get__(self):
            return self.doc[self.c.l_edge]

    property right_edge:
        """The rightmost token of this token's syntactic descendents.

        RETURNS (Token): The last token such that `self.is_ancestor(token)`.
        """
        def __get__(self):
            return self.doc[self.c.r_edge]

    property ancestors:
        """A sequence of this token's syntactic ancestors.

        YIELDS (Token): A sequence of ancestor tokens such that
            `ancestor.is_ancestor(self)`.
        """
        def __get__(self):
            cdef const TokenC* head_ptr = self.c
            # guard against infinite loop, no token can have
            # more ancestors than tokens in the tree
            cdef int i = 0
            while head_ptr.head != 0 and i < self.doc.length:
                head_ptr += head_ptr.head
                yield self.doc[head_ptr - (self.c - self.i)]
                i += 1

    def is_ancestor(self, descendant):
        """Check whether this token is a parent, grandparent, etc. of another
        in the dependency tree.

        descendant (Token): Another token.
        RETURNS (bool): Whether this token is the ancestor of the descendant.
        """
        if self.doc is not descendant.doc:
            return False
        return any(ancestor.i == self.i for ancestor in descendant.ancestors)

    property head:
        """The syntactic parent, or "governor", of this token.

        RETURNS (Token): The token predicted by the parser to be the head of
            the current token.
        """
        def __get__(self):
            return self.doc[self.i + self.c.head]

        def __set__(self, Token new_head):
            # this function sets the head of self to new_head
            # and updates the counters for left/right dependents
            # and left/right corner for the new and the old head

            # do nothing if old head is new head
            if self.i + self.c.head == new_head.i:
                return

            cdef Token old_head = self.head
            cdef int rel_newhead_i = new_head.i - self.i

            # is the new head a descendant of the old head
            cdef bint is_desc = old_head.is_ancestor(new_head)

            cdef int new_edge
            cdef Token anc, child

            # update number of deps of old head
            if self.c.head > 0:  # left dependent
                old_head.c.l_kids -= 1
                if self.c.l_edge == old_head.c.l_edge:
                    # the token dominates the left edge so the left edge of
                    # the  head may change when the token is reattached, it may
                    # not change if the new head is a descendant of the current
                    # head

                    new_edge = self.c.l_edge
                    # the new l_edge is the left-most l_edge on any of the
                    # other dependents where the l_edge is left of the head,
                    # otherwise it is the head
                    if not is_desc:
                        new_edge = old_head.i
                        for child in old_head.children:
                            if child == self:
                                continue
                            if child.c.l_edge < new_edge:
                                new_edge = child.c.l_edge
                        old_head.c.l_edge = new_edge

                    # walk up the tree from old_head and assign new l_edge to
                    # ancestors until an ancestor already has an l_edge that's
                    # further left
                    for anc in old_head.ancestors:
                        if anc.c.l_edge <= new_edge:
                            break
                        anc.c.l_edge = new_edge

            elif self.c.head < 0:  # right dependent
                old_head.c.r_kids -= 1
                # do the same thing as for l_edge
                if self.c.r_edge == old_head.c.r_edge:
                    new_edge = self.c.r_edge

                    if not is_desc:
                        new_edge = old_head.i
                        for child in old_head.children:
                            if child == self:
                                continue
                            if child.c.r_edge > new_edge:
                                new_edge = child.c.r_edge
                        old_head.c.r_edge = new_edge

                    for anc in old_head.ancestors:
                        if anc.c.r_edge >= new_edge:
                            break
                        anc.c.r_edge = new_edge

            # update number of deps of new head
            if rel_newhead_i > 0:  # left dependent
                new_head.c.l_kids += 1
                # walk up the tree from new head and set l_edge to self.l_edge
                # until you hit a token with an l_edge further to the left
                if self.c.l_edge < new_head.c.l_edge:
                    new_head.c.l_edge = self.c.l_edge
                    for anc in new_head.ancestors:
                        if anc.c.l_edge <= self.c.l_edge:
                            break
                        anc.c.l_edge = self.c.l_edge

            elif rel_newhead_i < 0:  # right dependent
                new_head.c.r_kids += 1
                # do the same as for l_edge
                if self.c.r_edge > new_head.c.r_edge:
                    new_head.c.r_edge = self.c.r_edge
                    for anc in new_head.ancestors:
                        if anc.c.r_edge >= self.c.r_edge:
                            break
                        anc.c.r_edge = self.c.r_edge

            # set new head
            self.c.head = rel_newhead_i

    property conjuncts:
        """A sequence of coordinated tokens, including the token itself.

        YIELDS (Token): A coordinated token.
        """
        def __get__(self):
            """Get a list of conjoined words."""
            cdef Token word
            if 'conjuncts' in self.doc.user_token_hooks:
                yield from self.doc.user_token_hooks['conjuncts'](self)
            else:
                if self.dep_ != 'conj':
                    for word in self.rights:
                        if word.dep_ == 'conj':
                            yield word
                            yield from word.conjuncts

    property ent_type:
        """RETURNS (uint64): Named entity type."""
        def __get__(self):
            return self.c.ent_type

        def __set__(self, ent_type):
            self.c.ent_type = ent_type

    property ent_iob:
        """IOB code of named entity tag. `1="I", 2="O", 3="B"`. 0 means no tag
        is assigned.

        RETURNS (uint64): IOB code of named entity tag.
        """
        def __get__(self):
            return self.c.ent_iob

    property ent_type_:
        """RETURNS (unicode): Named entity type."""
        def __get__(self):
            return self.vocab.strings[self.c.ent_type]

        def __set__(self, ent_type):
            self.c.ent_type = self.vocab.strings.add(ent_type)

    property ent_iob_:
        """IOB code of named entity tag. "B" means the token begins an entity,
        "I" means it is inside an entity, "O" means it is outside an entity,
        and "" means no entity tag is set.

        RETURNS (unicode): IOB code of named entity tag.
        """
        def __get__(self):
            iob_strings = ('', 'I', 'O', 'B')
            return iob_strings[self.c.ent_iob]

    property ent_id:
        """RETURNS (uint64): ID of the entity the token is an instance of,
            if any.
        """
        def __get__(self):
            return self.c.ent_id

        def __set__(self, hash_t key):
            self.c.ent_id = key

    property ent_id_:
        """RETURNS (unicode): ID of the entity the token is an instance of,
            if any.
        """
        def __get__(self):
            return self.vocab.strings[self.c.ent_id]

        def __set__(self, name):
            self.c.ent_id = self.vocab.strings.add(name)

    property whitespace_:
        """RETURNS (unicode): The trailing whitespace character, if present.
        """
        def __get__(self):
            return ' ' if self.c.spacy else ''

    property orth_:
        """RETURNS (unicode): Verbatim text content (identical to
            `Token.text`). Exists mostly for consistency with the other
            attributes.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.orth]

    property lower_:
        """RETURNS (unicode): The lowercase token text. Equivalent to
            `Token.text.lower()`.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.lower]

    property norm_:
        """RETURNS (unicode): The token's norm, i.e. a normalised form of the
            token text. Usually set in the language's tokenizer exceptions or
            norm exceptions.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.norm]

    property shape_:
        """RETURNS (unicode): Transform of the tokens's string, to show
            orthographic features. For example, "Xxxx" or "dd".
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.shape]

    property prefix_:
        """RETURNS (unicode): A length-N substring from the start of the token.
            Defaults to `N=1`.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.prefix]

    property suffix_:
        """RETURNS (unicode): A length-N substring from the end of the token.
            Defaults to `N=3`.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.suffix]

    property lang_:
        """RETURNS (unicode): Language of the parent document's vocabulary,
            e.g. 'en'.
        """
        def __get__(self):
            return self.vocab.strings[self.c.lex.lang]

    property lemma_:
        """RETURNS (unicode): The token lemma, i.e. the base form of the word,
            with no inflectional suffixes.
        """
        def __get__(self):
            if self.c.lemma == 0:
                return self.vocab.morphology.lemmatizer.lookup(self.orth_)
            else:
                return self.vocab.strings[self.c.lemma]

        def __set__(self, unicode lemma_):
            self.c.lemma = self.vocab.strings.add(lemma_)

    property pos_:
        """RETURNS (unicode): Coarse-grained part-of-speech tag."""
        def __get__(self):
            return parts_of_speech.NAMES[self.c.pos]

    property tag_:
        """RETURNS (unicode): Fine-grained part-of-speech tag."""
        def __get__(self):
            return self.vocab.strings[self.c.tag]

        def __set__(self, tag):
            self.tag = self.vocab.strings.add(tag)

    property dep_:
        """RETURNS (unicode): The syntactic dependency label."""
        def __get__(self):
            return self.vocab.strings[self.c.dep]

        def __set__(self, unicode label):
            self.c.dep = self.vocab.strings.add(label)

    property is_oov:
        """RETURNS (bool): Whether the token is out-of-vocabulary."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_OOV)

    property is_stop:
        """RETURNS (bool): Whether the token is a stop word, i.e. part of a
            "stop list" defined by the language data.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_STOP)

    property is_alpha:
        """RETURNS (bool): Whether the token consists of alpha characters.
            Equivalent to `token.text.isalpha()`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_ALPHA)

    property is_ascii:
        """RETURNS (bool): Whether the token consists of ASCII characters.
            Equivalent to `[any(ord(c) >= 128 for c in token.text)]`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_ASCII)

    property is_digit:
        """RETURNS (bool): Whether the token consists of digits. Equivalent to
            `token.text.isdigit()`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_DIGIT)

    property is_lower:
        """RETURNS (bool): Whether the token is in lowercase. Equivalent to
            `token.text.islower()`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_LOWER)

    property is_upper:
        """RETURNS (bool): Whether the token is in uppercase. Equivalent to
            `token.text.isupper()`
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_UPPER)

    property is_title:
        """RETURNS (bool): Whether the token is in titlecase. Equivalent to
            `token.text.istitle()`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_TITLE)

    property is_punct:
        """RETURNS (bool): Whether the token is punctuation."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_PUNCT)

    property is_space:
        """RETURNS (bool): Whether the token consists of whitespace characters.
            Equivalent to `token.text.isspace()`.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_SPACE)

    property is_bracket:
        """RETURNS (bool): Whether the token is a bracket."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_BRACKET)

    property is_quote:
        """RETURNS (bool): Whether the token is a quotation mark."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_QUOTE)

    property is_left_punct:
        """RETURNS (bool): Whether the token is a left punctuation mark."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_LEFT_PUNCT)

    property is_right_punct:
        """RETURNS (bool): Whether the token is a left punctuation mark."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_RIGHT_PUNCT)

    property is_currency:
        """RETURNS (bool): Whether the token is a currency symbol."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, IS_CURRENCY)

    property like_url:
        """RETURNS (bool): Whether the token resembles a URL."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, LIKE_URL)

    property like_num:
        """RETURNS (bool): Whether the token resembles a number, e.g. "10.9",
            "10", "ten", etc.
        """
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, LIKE_NUM)

    property like_email:
        """RETURNS (bool): Whether the token resembles an email address."""
        def __get__(self):
            return Lexeme.c_check_flag(self.c.lex, LIKE_EMAIL)



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.ru.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    #Translations from English:
    "Apple рассматривает возможность покупки стартапа из Соединённого Королевства за $1 млрд",
    "Беспилотные автомобили перекладывают страховую ответственность на производителя",
    "В Сан-Франциско рассматривается возможность запрета роботов-курьеров, которые перемещаются по тротуару",
    "Лондон — это большой город в Соединённом Королевстве",
    
    #Native Russian sentences:
    #Colloquial:
    "Да, нет, наверное!",#Typical polite refusal
    "Обратите внимание на необыкновенную красоту этого города-героя Москвы, столицы нашей Родины!",#From a tour guide speech
    
    #Examples of Bookish Russian:
    #Quote from "The Golden Calf"
    "Рио-де-Жанейро — это моя мечта, и не смейте касаться её своими грязными лапами!",
    
    #Quotes from "Ivan Vasilievich changes his occupation"
    "Ты пошто боярыню обидел, смерд?!!",
    "Оставь меня, старушка, я в печали!",
    
    #Quotes from Dostoevsky:
    "Уж коли я, такой же, как и ты, человек грешный, над тобой умилился и пожалел тебя, кольми паче бог", 
    "В мечтах я нередко, говорит, доходил до страстных помыслов о служении человечеству и может быть действительно пошел бы на крест за людей, если б это вдруг как-нибудь потребовалось, а между тем я двух дней не в состоянии прожить ни с кем в одной комнате, о чем знаю из опыта",
    "Зато всегда так происходило, что чем более я ненавидел людей в частности, тем пламеннее становилась любовь моя к человечеству вообще",
    
    #Quotes from Chekhov:
    "Ненужные дела и разговоры всё об одном отхватывают на свою долю лучшую часть времени, лучшие силы, и в конце концов остается какая-то куцая, бескрылая жизнь, какая-то чепуха, и уйти и бежать нельзя, точно сидишь в сумасшедшем доме или в арестантских ротах!",
    
    #Quotes from Turgenev:
    "Нравится тебе женщина, старайся добиться толку; а нельзя — ну, не надо, отвернись — земля не клином сошлась",
    "Узенькое местечко, которое я занимаю, до того крохотно в сравнении с остальным пространством, где меня нет и где дела до меня нет; и часть времени, которую мне удастся прожить, так ничтожна перед вечностью, где меня не было и не будет...",
    
    #Quotes from newspapers:
    #Komsomolskaya Pravda:
    "На заседании президиума правительства Москвы принято решение присвоить статус инвестиционного приоритетного проекта города Москвы киностудии Союзмультфильм",
    "Глава Минобороны Сергей Шойгу заявил, что обстановка на этом стратегическом направлении требует непрерывного совершенствования боевого состава войск",
    #Argumenty i Facty:
    "На реплику лже-Говина — дескать, он (Волков) будет лучшим революционером — Стамп с энтузиазмом ответил: Непременно!",
]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from ...attrs import LIKE_NUM

_num_words = ['μηδέν', 'ένας', 'δυο', 'δυό', 'τρεις', 'τέσσερις', 'πέντε',
              'έξι', 'εφτά', 'επτά', 'οκτώ', 'οχτώ',
              'εννιά', 'εννέα', 'δέκα', 'έντεκα', 'ένδεκα', 'δώδεκα',
              'δεκατρείς', 'δεκατέσσερις', 'δεκαπέντε', 'δεκαέξι', 'δεκαεπτά',
              'δεκαοχτώ', 'δεκαεννέα', 'δεκαεννεα', 'είκοσι', 'τριάντα',
              'σαράντα', 'πενήντα', 'εξήντα', 'εβδομήντα', 'ογδόντα',
              'ενενήντα', 'εκατό', 'διακόσιοι', 'διακόσοι', 'τριακόσιοι',
              'τριακόσοι', 'τετρακόσιοι', 'τετρακόσοι', 'πεντακόσιοι',
              'πεντακόσοι', 'εξακόσιοι', 'εξακόσοι', 'εφτακόσιοι', 'εφτακόσοι',
              'επτακόσιοι', 'επτακόσοι', 'οχτακόσιοι', 'οχτακόσοι',
              'οκτακόσιοι', 'οκτακόσοι', 'εννιακόσιοι', 'χίλιοι', 'χιλιάδα',
              'εκατομμύριο', 'δισεκατομμύριο', 'τρισεκατομμύριο', 'τετράκις',
              'πεντάκις', 'εξάκις', 'επτάκις', 'οκτάκις', 'εννεάκις', 'ένα',
              'δύο', 'τρία', 'τέσσερα', 'δις', 'χιλιάδες']


def like_num(text):
    text = text.replace(',', '').replace('.', '')
    if text.isdigit():
        return True
    if text.count('/') == 1:
        num, denom = text.split('/')
        if num.isdigit() and denom.isdigit():
            return True
    if text.count('^') == 1:
        num, denom = text.split('^')
        if num.isdigit() and denom.isdigit():
            return True
    if text.lower() in _num_words or text.lower().split(' ')[0] in _num_words:
        return True
    if text in _num_words:
        return True
    return False


LEX_ATTRS = {
    LIKE_NUM: like_num
}



# coding: utf8
from __future__ import unicode_literals
from ..util import add_vecs_to_vocab, get_doc
import pytest

@pytest.fixture
def vectors():
    return [("a", [1, 2, 3]), ("letter", [4, 5, 6])]

@pytest.fixture
def vocab(en_vocab, vectors):
    add_vecs_to_vocab(en_vocab, vectors)
    return en_vocab

def test_issue2219(vocab, vectors):
    [(word1, vec1), (word2, vec2)] = vectors
    doc = get_doc(vocab, words=[word1, word2])
    assert doc[0].similarity(doc[1]) == doc[1].similarity(doc[0])



# coding: utf8
from __future__ import unicode_literals


DETS_IRREG = {
    "aucune": ("aucun",),
    "ces": ("ce",),
    "cet": ("ce",),
    "cette": ("ce",),
    "cents": ("cent",),
    "certaines": ("certains",),
    "différentes": ("différents",),
    "diverses": ("divers",),
    "la": ("le",),
    "les": ("le",),
    "l'": ("le",),
    "laquelle": ("lequel",),
    "lesquelles": ("lequel",),
    "lesquels": ("lequel",),
    "leurs": ("leur",),
    "mainte": ("maint",),
    "maintes": ("maint",),
    "maints": ("maint",),
    "ma": ("mon",),
    "mes": ("mon",),
    "nos": ("notre",),
    "nulle": ("nul",),
    "nulles": ("nul",),
    "nuls": ("nul",),
    "quelle": ("quel",),
    "quelles": ("quel",),
    "quels": ("quel",),
    "quelqu'": ("quelque",),
    "quelques": ("quelque",),
    "sa": ("son",),
    "ses": ("son",),
    "telle": ("tel",),
    "telles": ("tel",),
    "tels": ("tel",),
    "ta": ("ton",),
    "tes": ("ton",),
    "tous": ("tout",),
    "toute": ("tout",),
    "toutes": ("tout",),
    "des": ("un",),
    "une": ("un",),
    "vingts": ("vingt",),
    "vos": ("votre",)
}



# coding: utf8
from __future__ import unicode_literals

# Here we only want to include the absolute most common words. Otherwise,
# this list would get impossibly long for German – especially considering the
# old vs. new spelling rules, and all possible cases.


_exc = {
    "daß": "dass"
}


NORM_EXCEPTIONS = {}

for string, norm in _exc.items():
    NORM_EXCEPTIONS[string] = norm
    NORM_EXCEPTIONS[string.title()] = norm



# coding: utf8

#structure copied from the English lemmatizer
from __future__ import unicode_literals

from .lookup import LOOKUP
from ._adjectives_wordforms import ADJECTIVES_WORDFORMS
from ._adverbs_wordforms import ADVERBS_WORDFORMS
from ._nouns_wordforms import NOUNS_WORDFORMS
from ._verbs_wordforms import VERBS_WORDFORMS
from ._lemma_rules import ADJECTIVE_RULES, NOUN_RULES, VERB_RULES, PUNCT_RULES
from ._verbs import VERBS
from ._nouns import NOUNS
from ._adjectives import ADJECTIVES
from ._adverbs import ADVERBS

LEMMA_INDEX = {'adj': ADJECTIVES, 'adv': ADVERBS, 'noun': NOUNS, 'verb': VERBS}

LEMMA_EXC = {'adj': ADJECTIVES_WORDFORMS, 'adv': ADVERBS_WORDFORMS, 'noun': NOUNS_WORDFORMS,
             'verb': VERBS_WORDFORMS}

LEMMA_RULES = {'adj': ADJECTIVE_RULES, 'noun': NOUN_RULES, 'verb': VERB_RULES,
               'punct': PUNCT_RULES}


# coding: utf8
from __future__ import unicode_literals


ADJECTIVES = set("""
صفر
صفرم
صفر‌ام
یک
یکم
یک‌ام
اول
نخست
دو
دوم
دو‌ام
سه
سوم
چهار
چهارم
پنج
پنجم
پنج‌ام
شش
ششم
شش‌ام
هفت
هفتم
هفت‌ام
هشت
هشتم
هشت‌ام
نه
نهم
نه‌ام
ده
دهم
ده‌ام
یازده
یازدهم
یازده‌ام
دوازده
دوازدهم
دوازده‌ام
سیزده
سیزدهم
سیزده‌ام
چهارده
چهاردهم
چهارده‌ام
پانزده
پانزدهم
پانزده‌ام
پونزده
پونزدهم
پونزده‌ام
شانزده
شانزدهم
شانزده‌ام
شونزده
شونزدهم
شونزده‌ام
هفده
هفدهم
هفده‌ام
هجده
هجدهم
هجده‌ام
هیجده
هیجدهم
هیجده‌ام
نوزده
نوزدهم
نوزده‌ام
بیست
بیستم
بیست‌ام
سی
سی‌ام
چهل
چهلم
چهل‌ام
پنجاه
پنجاهم
پنجاه‌ام
شصت
شصتم
شصت‌ام
هفتاد
هفتادم
هفتاد‌ام
هشتاد
هشتادم
هشتاد‌ام
نود
نودم
نود‌ام
صد
صدم
یکصد
یکصدم
یک‌صد
یک‌صدم
دویست
دویستم
دویست‌ام
سیصد
سیصدم
سیصد‌ام
چهارصد
چهارصدم
چهارصد‌ام
پانصد
پانصدم
پانصد‌ام
پونصد
پونصدم
پونصد‌ام
ششصد
ششصدم
ششصد‌ام
شیشصد
شیشصدم
شیشصد‌ام
هفتصد
هفتصدم
هفتصد‌ام
هفصد
هفصدم
هفصد‌ام
هشتصد
هشتصدم
هشتصد‌ام
نهصد
نهصدم
نهصد‌ام
هزار
هزارم
هزار‌ام
میلیون
میلیونم
میلیون‌ام
میلیارد
میلیاردم
میلیارد‌ام
بیلیون
بیلیونم
بیلیون‌ام
بیلیارد
بیلیاردم
بیلیارد‌ام
تریلیون
تریلیونم
تریلیون‌ام
تریلیارد
تریلیاردم
تریلیارد‌ام
کوادریلیون
کوادریلیونم
کوادریلیون‌ام
کادریلیارد
کادریلیاردم
کادریلیارد‌ام
کوینتیلیون
کوینتیلیونم
کوینتیلیون‌ام
آبستن
آبکی
آبی
آب‌ندیده
آتی
آخته
آخر
آراسته
آرام
آرامش‌بخش
آرام‌بخشی
آرایشی
آرزومند
آرمانی
آرمیده
آزاد
آزادمنشی
آزاد‌ید
آزاردهنده
آزار‌دهنده
آزار‌دهندهٔ
آزرده
آزمایشگاهی
آسان
آسانگیر
آسمانی
آسوده
آسیایی
آسیب‌دیده
آسیب‌پذیر
آشامیدنی
آشفته
آشنا
آشنای
آشوب‌طلب
آشکار
آشکاری
آشکار‌شده
آفریقایی
آلاینده
آلمانی
آلوده
آلومینیومی
آماده
آمادهٔ
آماری
آمده
آمریکایی
آموزشی
آموزنده
آمیخته
آنالوگ
آنتی‌سپتیک
آنتی‌وایرال
آنی
آهنی
آورده
آویزان
آکادمیک
آکنده
آگاه
آینده
آینده‌نگر
آینه‌کردار
آیینی
ائتلافی
ابتدایی
ابداعی
ابدی
ابزار‌خورده
ابطال‌شدنی
ابله
اتفاقیه
اتمی
اثبات‌ناپذیر
اثرگذار
اثر‌گذار
اجبارآمیز
اجباری
اجتماعی
اجتماعی‌
اجدادی
اجدادی‌
اجرایی
اجمالی
احتمالی
احساساتی
احساسی
اختصاصی
اختلافی
اخلاقی
اخیر
اداری
ادبی
ادب‌شناس
ادراری
ادعایی
ادیب
ارائه‌شده
اراکی
ارتباطی
ارتجاعی
ارجمند
اردنی
اردو
ارزان
ارزشمند
ارزشمندی
ارزشگرایانه
ارزشی
ارزنده
ارزندهٔ
ارزی
ارزی‌
ارشد
ارضی
اروماتیک
اروپائی
اروپایی
ارگانیک
از‌بین‌برنده
از‌دست‌رفته
از‌هم‌گسسته
اساسی
اساطیری
استادانه
استالینیست
استامپی
استاندارد
استبدادی
استحاله
استخراجی
استخوانی
استراتژیک
استرالیایی
استقلال‌یافته
استوار
استیلیزه
اسرع
اسفل
اسف‌انگیز
اسلامگرای
اسلامی
اسلامی‌
اسلام‌گرا
اسلوموشن
اسمی
اسپانیایی
اشتباهی
اشرافی
اشغالگر
اشکانی
اصفهانی
اصلاحی
اصلاح‌طلب
اصلاح‌طلبانه
اصلاح‌گر
اصلاح‌گرا
اصلاح‌گرایانه
اصلی
اصولگرا
اصولی
اصیل
اضافه
اضافه‌تری
اضافی
اضطراری
اطلاعاتی
اطمینان‌بخش
اعتباری
اعتقادی
اعزامی
اعطایی
اعظم
اعلام‌شده
اعلای
اعم
اعمال‌شده
اغلب
افازیک
افراطی
افزایشی
افزوده
افزودهٔ
افزون
افسرده
افضل
افغانی
اقتصادی
اقصی
اقلیدسی
اقلیمی
التقاطی
التهابی
الجزایری
الحاقی
الزامی
الهی
الکترونیک
الکترونیکی
الکلی
الیت
امامی
امان
امانتدار
امانتی
امدادرسانی
امدادگر
امدادی
امروزی
امریکایی
امن
امنیتی
اموی
امکان‌پذیر
امیدوار
امیدوارکننده
انباشته
انبوه
انتخاباتی
انتخابی
انتخابیه
انتظامی
انتقادی
انجام‌شده
انجام‌گرفته
انحرافی
انداخته
اندوهناک
اندک
اندکی
اندیشمندی
اندیشگی
انرژی‌بخش
انرژی‌زا
انرژی‌زایی
انسانی
انسانیِ
انسان‌دوست
انعطاف‌ناپذیر
انفرادی
انفعالی
انقلابی
انکارناپذیر
انگاشته
انگشتری
انگلی
انگلیسی
انگیزشی
انیمیشنی
اهانت‌آمیز
اهانت‌آمیزی
اهدایی
اهلی
اهم
اول
اولی
اولیه
اوکراینی
اکبر
اکتشافی
اکتیو
اکثر
اکراه
اکرم
اکسیده
اکشن
اکلیلی
اکونومیک
ایالتی
ایتالیایی
ایجاد‌شده
ایداولوژیک
ایرانی
ایرانی‌
ایرانی‌نژاد
ایران‌زا
ایستاده
ایمانی
ایمنی
اینترنتی
باارزش
بااطلاع
باایمان
باتجربه
باتقوای
باحیا
باختری
بادی
بارانی
باردار
بارداری
بارزی
باریکی
باز
بازاری
بازشماری‌شده
بازمانده
بازنشسته
بازگشتی
باستان
باستانی
باشکوهی
باصفا
باطل
باطلی
باظرافت
باعث
باغی
باقی
باقیمانده
باقی‌مانده
بالاتری
بالادستی
بالای
بالایی
بالغ
بالقوه
بانشاط
بانکی
باوفا
باکتریایی
با‌ادب
با‌ارزش
با‌ارزشی
با‌اقتدار
با‌انصاف
با‌تجربه
با‌تدبیر
با‌سابقه
با‌شکوهی
با‌عزت
با‌فکر
با‌نمک
بجا
بحث‌انگیز
بحرانی
بحران‌ساز
بخشنده
بد
بدجنس
بدخط
بدخیم
بدنام
بدنی
بدگمان
بدی
بدیع
بدیعی
بدیل
بدیهی
برآورده
برادرانه
برافراشته
براق
برانداز
برانگیزاننده
برجسته
برجستهٔ
برجسته‌یی
برحذر
برخوردار
بردبار
برطرف
برعکس
برفکی
برق
برقرار
برنامه‌پرکن
برنده
برنزی
برهم
برهنه
برهنه‌
بروسلایی
برون‌زا
برون‌ساحلی
برون‌مرزی
برپا
برگ
برگردانده
برگزار
برگزیده
برگشتی
برگ‌ریزان
بریده
بزرگ
بزرگش
بزرگوار
بزرگی
بستری
بسته
بسنده
بسیار
بسیاری
بشاش
بشردوست
بشردوستانه
بشقابی
بصری
بطای
بعد
بعدی
بعید
بلند
بلندمدت
بلندپایه
بلندی
بلورینی
بلوچ
بلیغ
بندری
بنیانی
به
بهاری
بهتان
بهتری
بهت‌انگیز
بهت‌زده
بهداشتی
بهره‌مند
بهشتی
بهینه
بهینه‌ساز
به‌روز
به‌قاعده
به‌موقع
به‌نام
به‌کار‌رفته
بوده
بومی
بچه‌دار
بیابانی
بیدار
بیرونی
بیزار
بیستم
بیش
بیشتری
بیشماری
بیضی‌شکل
بیمار
بیماری
بیماری‌زا
بیماری‌زای
بین‌الملل
بین‌المللی
بیهوده
بیوشیمیایی
بیکار
بیگانه
بیگاه
بیگناه
بی‌آب
بی‌آزار
بی‌اثر
بی‌اختیار
بی‌ارزش
بی‌اساس
بی‌اطلاع
بی‌اعتبار
بی‌انتها
بی‌بهره‌
بی‌تاب
بی‌تفاوت
بی‌ثبات
بی‌ثمر
بی‌جان
بی‌حدومرز
بی‌حساب
بی‌خار
بی‌خبر
بی‌خطر
بی‌خود
بی‌دریغ
بی‌دفاع
بی‌ربط
بی‌رویه
بی‌زار
بی‌زبانی
بی‌سابقه
بی‌سروته‌شده
بی‌سواد
بی‌شاخ
بی‌شمار
بی‌صاحب
بی‌طاقت
بی‌طرف
بی‌طپش
بی‌فایده
بی‌مسئل
بی‌معنا
بی‌معنی
بی‌مهار
بی‌مورد
بی‌نتیجه
بی‌نظیر
بی‌نهایت
بی‌نی
بی‌نیاز
بی‌پاسخ
بی‌پایان
بی‌پایه
بی‌پروا
بی‌کار
بی‌گناه
بی‌گناهی
تأثیربرانگیز
تأثیرگذار
تألیفی
تئوریک
تابستانی
تابعه
تابناک
تاریخی
تاریک
تاریکی
تازه
تازه‌تری
تازه‌داماد
تازه‌کار
تازه‌یی
تایوانی
تایپه
تاییدشده
تبتی
تبلیغاتی
تبلیغاتیِ
تبهکاری
تجارتی
تجاری
تجدیدنظرطلب
تجدیدنظر‌طلبانه
تجدید‌کننده
تجربه‌شده
تجربی
تجسمی
تجملی
تجویزی
تحتانی
تحریریه
تحریک‌کننده
تحسین‌برانگیز
تحسین‌کننده
تحصیلی
تحصیل‌کرده
تحقیقاتی
تحقیقی
تحلیلی
تحمیلی
تخصصی
تخیلی
تخیلی‌
تدارکاتی
تدریجی
ترافیکی
ترانزیتی
ترسان
ترسناک
ترسو
ترشحی
ترفیع
ترقیخواهانه
ترقی‌خواهانه
تزئینی
تزریقی
تشخیصی
تشریفاتی
تشنه
تشکیلاتی
تشکیلاتی‌
تصادفی
تصفیه‌کننده
تصویبی
تصویب‌شده
تصویری
تضییع‌کننده
تعجب‌انگیز
تعطیل
تعطیل‌شده
تعمدی
تعمیرکار
تعیین‌شده
تعیین‌کننده
تفریحی
تقریبی
تقویتی
تلخ
تلخ‌زبان
تلفنی
تلف‌شده
تلویزیونی
تمام
تمام‌شدنی
تمام‌شده
تمام‌لبه
تمام‌نما
تمام‌وقت
تمدنی
تمیز
تناقض‌آمیز
تناور
تند
تندرو
تندروی
تنظیم‌شده
تنظیم‌کننده
تنفسی
تنها
تنومند
تنک
تنگ
تنگاتنگ
تنگ‌نظر
تهرانی
تهیه‌شده
تهیه‌کننده
تهی‌دست
توأم
توافقی
توانا
توانمند
توجیه‌پذیر
توحیدی
تورمی
تورم‌زا
توریستی
توسعه‌یافته
توقیف‌شده
تولیدکننده
تولیدگرا
تولیدی
تو‌در‌تو
تکان‌دهنده
تکراری
تکنولوژیکی
تکنیکی
تک‌ساحتی
تک‌پای
تک‌چشم
تیرانداز
تیره
تیز
تیزی
تیمی
ثابت
ثانوی
جاار
جاذبه
جاری
جاسوسی
جالب
جالبی
جامد
جامع
جامعه‌شناختی
جانوری
جانی
جان‌به‌لب
جاودانه
جاودانی
جاویدان
جبری
جدا
جداگانه
جدای
جدی
جدید
جدیدی
جدی‌تری
جذاب
جذابی
جرمزا
جزء
جزئی
جزایی
جسته
جسمانی
جسمی
جعلی
جفت
جمع
جمعی
جناحی
جناقی
جنایتکار
جنایی
جنبی
جنت‌نشان
جنجالی
جنجال‌انگیز
جنوبی
جنگلی
جنگی
جهانی
جهان‌شمول
جهان‌نمای
جوان
جوانانه
جوانتری
جوانمرد
جوانی
جور
جوشان
جوی
جویا
جیبی
حائز
حاد
حادث
حاصله
حاضر
حاوی
حاکم
حاکی
حتمی
حتمی‌الاجرا
حجمی
حداکثر
حداکثری
حذفی
حذف‌شده
حرارتی
حرام
حرمت‌شکن
حریف
حزبی
حسابی
حساس
حساسی
حساسیت‌زا
حساسیت‌زای
حسنه
حضوری
حقه
حقوقدان
حقوقی
حقیر
حقیقی
حق‌شناس
حق‌طلب
حلال
حلقومی
حلق‌آویز
حلیم
حمل
حوزوی
حکمت‌آمیز
حکومتی
حکیم
حیرت‌آور
حیرت‌انگیز
حیرت‌برانگیز
حیوانی
خائن
خارجه
خارجی
خارج‌شده
خاص
خاصی
خاضع
خاطرجمع
خاطی
خالی
خام
خاموش
خانوادگی
خانگی
خاکستری
خاکی
خبره
خبری
خجالتی
خدایی
خدشه‌دار
خدماتی
خراب
خرامان
خردسال
خرده‌فرهنگی
خرسند‌کننده
خرم
خرمی
خروجی
خزنده
خسارت‌دیده
خسته
خسته‌
خستگی‌ناپذیر
خشن
خشنود
خشنی
خشونت‌بار
خشک
خصوصی
خطاپوش
خطرناک
خطرناکی
خطی
خطیب
خطیر
خطیری
خط‌دهنده
خفه
خلاصه
خلاف
خلاق
خم
خمیده
خنثی
خندان
خنک
خواسته
خوب
خوبی
خودسر
خودکار
خودکامه
خودی
خوش
خوشایند
خوشایندی
خوشبخت
خوشبو
خوشبو‌کننده
خوشحال
خوشرفتار
خوشمزه
خوشوقت
خوشگوار
خوشی
خوش‌اخلاق
خوش‌خیم
خوش‌دست
خوش‌دهان
خوش‌رفتار
خوش‌سخن
خوش‌فرجام
خوش‌فکر
خوش‌نام
خونرنگ
خونسرد
خونی
خیال‌پرداز
خیال‌پردازانه
خیانتکار
خیر
خیراندیش
خیرخواه
خیره
خیره‌کننده
خیری
خیس
دائر
دائم
دائمی
داخلی
دارا
دارای
دارویی
داستانی
داغول‌باز
دامی
دانشجویی
دانشمند
دانشگاهی
داوطلب
داوطلبانه
دایر
دایمی
دخیل
درآمده
درآورده
دراز
درازمدت
درازی
دربرگیرنده
درحال‌گذار
درختی
درخشان
درخشانی
درخور
دردناک
درست
درستکردار
درستی
درست‌کردار
درشت
درصدد
درفشان
درمانی
دروغ
دروغگو
دروغگوی
درونی
درون‌استانی
درگیر
دریافتی
دریایی
در‌آمده
در‌برگیرندهٔ
در‌حال‌توسعه
در‌خور
در‌رفته
در‌هم‌تنیدهٔ
دستباف
دستجمعی
دسته‌جمعی
دستوری
دستگیر
دستی
دست‌آموز
دست‌افشان
دست‌به‌کار
دست‌ساز
دست‌کمی
دست‌یافتنی‌
دشمن
دشوار
دشواری
دعوت‌شده
دفاعی
دفعی
دقیق
دقیق‌تری
دلاری
دلالی
دلخواه
دلرحم
دلسوز
دلهره‌آور
دلپذیر
دلگشا
دلگیر
دل‌زندهٔ
دل‌مشغول
دموکراتیک
دمکراتیک
دهشتناک
دهم
دهمی
دوازدهم
دوجانبه
دوخته
دودوزه‌باز
دور
دورافتاده
دوراندیش
دوردست
دور‌دست
دوست
دوستانه
دوست‌داشتنی
دوسوی
دوقسمتی
دوقلو
دولتی
دوم
دومی
دوم‌خردادی
دونفره
دوگانه
دو‌جانبه
دو‌زبانه
دو‌فوریتی
دچار
دکتری
دگر
دگراندیش
دگرگون
دیابتی
دیجیتال
دیجیتالی
دیر
دیری
دیر‌فهم
دیر‌قدمت
دیم
دینی
دین‌ستیزی
دیواری
دیوانه‌
دیپلماتیک
دیگر
دیگری
دیگه
ذهنی
ذوب‌شده
ذکر‌شده
ذکور
ذیربط
ذیل
رإف
راحت
راحت‌تری
راحل
رادیویی
رادیو‌اکتیو
رازدار
راست
راستگو
راستی
راسخ
راضی
رافضی
راهبردی
راهنما
راهگشای
رایج
رایگان
ربات
ربانی
رزمی
رسای
رسمی
رسوا
رسیده
رسیدگی‌کننده
رشتی
رشد‌دهنده
رفتاری
رفته
رفیع
رقابتی
رقص‌وار
رنجور
رنگارنگ
رنگارنگی
رهگشا
روا
روائی
روان
روانه
روانی
روبرو
روبه‌رو
روحانی
روحی
روحی‌
روراست
روز
روزانه
روزمره
روزه
روزی‌گذار
روس
روستایی
روسی
روشن
روشنفکر
روشنفکری
روشنی
روند
رویایی
ریاضی‌دان
ریالی
ریخته‌شده
ریختی
ریز
ریزبافت
ریشه‌دار
ریشه‌کن
ریوی
ریگی
رییسه
زار
زاهدانه
زاید
زایل
زبانی
زبر
زبردست
زخمی
زرنگ
زرنگی
زرینی
زرین‌فام
زشت
زشتی
زمانی
زمان‌بندی‌شده
زمان‌شناسی
زمستانی
زمینه‌ساز
زمینی
زنانه
زنانهٔ
زنانه‌یی
زنبوری
زنبوری‌
زنجانی
زنجیر‌شده
زندانی
زنده
زنده‌
زندگی‌دوست
زننده
زنگباری
زود
زودباور
زودرس
زودگذر
زود‌هضمی
زیاد
زیادی
زیانبخش
زیان‌بار
زیبا
زیباشناختی
زیبای
زیبایی
زیر
زیربنایی
زیرزمینی
زیرلاکی
زیرک
زیرک‌
زیست‌محیطی
زینتی
زیگ‌زاگ‌گونه
سابق
ساحلی
ساختاری
ساختمانی
ساخته
ساختهٔ
ساخته‌شده
ساختگی
ساده
ساده‌
سازمانی
سازمان‌یافته
سازنده
سازگار
ساعته
ساعتی
ساقط
ساقط‌شده
سالانه
سالم
سالمند
ساله
سالهٔ
ساله‌
سالگی
سالیانه
سامانی
سانتی‌متری
ساکت
ساکن
سبز
سبک
سبکبال
سبکسر
ستایشی
ستاینده
ستمدیده‌یی
ستم‌شاهی
سحرانگیز
سخت
سختگیر
سرانه
سربلند
سربه‌نیست
سرخ
سرخوش
سرد
سردسیری
سرزده
سرسخت
سرشار
سرشناس
سرطانی
سرطان‌زا
سرمست
سرنگون
سروده
سرپایی
سرکش
سرکوب‌کننده
سرگردان
سرگرم
سرگرم‌کننده
سری
سریع
سریعتری
سر‌درگم
سست
سستی
سطحی
سطحی‌نگر
سعودی
سفالی
سفت
سفید
سفیدرنگ
سفیدپوست
سفیدپوش
سلبی
سلحشور
سلطانی
سلطنتی
سلولی
سلیم
سمعی
سمی
سنتی
سنت‌گرای
سنجیده
سنگی
سهامی
سهوی
سهیم
سه‌جلدی
سه‌ساله
سه‌فرسخی
سه‌گانه
سواره
سواری
سوای
سوخته
سودآور
سودازده
سودمند
سوسیالیستی
سوم
سومی
سوپرالیت
سوگوار
سپاسگزار
سپید
سکتاریسم
سکولار
سکولارولاییک
سگ‌باز
سیاحتی
سیار
سیاسی
سیاه
سیر
سیراب
سیرجانی
سیزدهم
سیستمی
سیفلیسی
سیلندری
سیل‌آسا
سیم‌گون
سینمایی
سیگاری
سی‌سؤالی
شاخص
شاخ‌دار
شاد
شاداب
شادمان
شاعرانه
شاعری
شاغل
شامخ
شامل
شانزدهم
شاهد
شایان
شایسته
شایستهٔ
شایع
شایعه‌پرداز
شبیه
شتابزده
شجاع
شجاعی
شخصی
شخصیتی
شدید
شدیدی
شراب‌خوار
شرعی
شرقی
شرقیِ
شرکت‌کننده
شریف
ششم
شش‌دانگی
شش‌نفره
شعاری
شعری
شعله‌ور
شغلی
شفاف
شفاهی
شل
شمالی
شمرده
شمسی
شناخته
شناخته‌شده
شناور
شنفته
شنوا
شنیده
شهرستانی
شهروندی
شهری
شهیر
شوخ
شور
شورایی
شوم
شوکه
شکاک
شکربار
شکسته
شکست‌خورده
شکفته
شکننده
شکوهمند
شکیبا
شک‌برانگیز
شگفت
شگفت‌آور
شگفت‌انگیز
شیبدار
شیرازی
شیطانی
شیفتهٔ
شیمیایی
شیک
صاحب‌نام
صادر
صادراتی
صادقانه
صاف
صالح
صبحگاهی
صبور
صحرائی
صحیح
صحیح‌تری
صدساله
صرف
صریح
صغیر
صلیبی
صمیمی
صنعتی
صنفی
صهیونیستی
صوتی
صورت‌گرفته
صیادی
ضبط‌شده
ضد
ضدارزش
ضداسلامی
ضدانقلابی
ضدعشقی
ضدمیگرن
ضد‌استبدادی
ضد‌استعماری
ضروری
ضروری‌
ضعیف
ضعیف‌النفس
ضمیمه
طاقت‌فرسا
طبقاتی
طبیعی
طلائی
طلایی
طلا‌نشانده
طنزآمیز
طولانی
طولانی‌مدت
طویلی
طیب
ظالمانه
ظاهری
ظریف
ظریفی
عاجز
عادتی
عادل
عادلانه
عادلِ
عادی
عارض
عارفانه
عاری
عاشق
عاشقانه
عاطفی
عاقل
عالی
عالیقدر
عالیه
عالی‌رتبه
عالی‌مقام
عام
عامه
عتیقه
عثمانی
عجیب
عربی
عربی‌شناس
عرب‌زبان
عرفانی
عرفی
عروقی
عریض
عزیز
عزیزِ
عشایری
عشقی
عشق‌انگیز
عصبانی
عصبی
عضو
عظام
عظیم
عظیمی
عظیم‌الجثه
عفونی
عقب‌افتاده
عقدکرده
عقلی
عقیدتی
علاقه‌مند
علمی
علمیه
علمیی
علنی
عمد
عمده
عمدهٔ
عمده‌یی
عمدی
عمرانی
عملگرا
عملی
عملیاتی
عملی‌
عمودی
عمومی
عمیق
عمیقی
عنبرین‌بو
عنود
عهد‌نگهدار
عیان
عینی
غائب
غاصب
غافل
غافلگیر
غافلگیرکننده
غایی
غبارآلود
غذاخوری
غذایی
غذایی‌
غربی
غربیِ
غرق
غرقِ
غره
غریب
غریبه
غریبی
غلط
غلطِ
غلیظ
غمزده
غم‌آباد
غم‌آلود
غم‌انگیز
غنی
غنی‌شده
غنی‌یی
غوطه‌ور
غول‌پیکر
غیرآهنی
غیراخلاقی
غیراستاندارد
غیراسلامی
غیراصلی
غیراصولی
غیرافسرده
غیراقلیدسی
غیرانقلابی
غیردوستانه
غیردولتی
غیردینی
غیررسمی
غیرسیاسی
غیرطبیعی
غیرعادلانه‌یی
غیرعادی
غیرعرب
غیرقانونی
غیرمارکسیست
غیرمترقبه
غیرمتعارف
غیرمحتمل
غیرمحقق
غیرمذهبی
غیرمستقیم
غیرمستقیمی
غیرمسلح
غیرمسیحی
غیرمعنوی
غیرملی
غیرمنتظره
غیرمنطقی
غیرمنقول
غیرموثق
غیرنظامی
غیرنفتی
غیرواقعی
غیرکلیشه
غیر‌انتخابی
غیر‌تخصصی
غیر‌رسمی
غیور
فئودالی
فاتح
فاحش
فارسی
فارسی‌دان
فارسی‌زبان
فارغ
فاسد
فاش
فاصل
فاضل
فاضله
فاطمی
فاقد
فانتزی
فانی
فداکار
فدرال
فراری
فراملی
فراموش
فرانسه
فرانسوی
فراوان
فراوانی
فراگیر
فراگیری
فربه
فرخنده
فردی
فردی‌
فرزانه
فرساینده
فرضی
فرعی
فرنگی
فرهنگی
فرهنگی‌
فرهنگ‌دوست
فروتن
فروخته
فروخته‌شده
فروزان
فروشندگی
فروگذار
فرو‌ریخته
فریبنده
فسخ‌شده
فسرده
فشرده
فصلی
فصیح
فضایی
فضایی‌
فضیلت‌ها
فعال
فعلی
فقهی
فقید
فقیر
فلزی
فلسفی
فلسفیی
فلکی
فنلاندی
فنی
فهمیده
فهیم
فوری
فوریتی
فوری‌
فوق
فوقانی
فوق‌الذکر
فوق‌العاده
فکری
فیروزه‌گون
فیزیولوژیکی
فیزیکی
فیلسوف
فیلیپینی
فی‌سبیل‌الله
قائل
قائم
قابل
قابل‌اتکاء
قابل‌توجه
قابل‌توجهی
قابل‌قبول
قابل‌مشاهده
قابل‌مقایسه
قابل‌نفی
قابل‌یادآوری
قادر
قاطبه
قاطع
قالبی‌شده
قانع
قانونمند
قانونی
قاچاقچی
قبطی
قبل
قبلی
قبلی‌
قبول
قبیله‌یی
قدرتمند
قدرتی
قدیم
قدیمی
قرآنی
قرض‌الحسنه
قرمز
قرمزی
قریب
قشنگ
قشنگتری
قشنگی
قضائیه
قضایی
قطعی
قفقازی
قلابی
قلبی
قلدری
قمری
قهرمانی
قهریه
قومی
قوی
قیصری
قیمتی
لاابالی
لاتینی‌نویس
لازم
لازمه
لاغر
لاینقطع
لاییک
لبالب
لبریز
لذت‌طلب
لطیف
لعابدار
لهجه‌دار
لوکس
لیبرال
م
مأثر
مؤثر
مؤثری
مؤمن
مؤمنی
مإدبانه
ماجراجو
مادام‌العمر
مادری
ماده
مادی
مارکسیست
مارکسیستی
مازاد
ماشینی
ماشین‌رو
ماشین‌زده
مالایلزم
مالی
مالیه
مالی‌
ماندنی
مانده
ماندگار
ماهانه
ماهری
ماهه
مایل
مبارز
مبارک
مبارکه
مبتذل
مبتلا
مبتنی
مبذول
مبرا
مبسوط
مبهمی
متأثر
متؤثر
متاثر
متبادر
متبحر
متبرک
متبسم
متجاوز
متجدد
متجددی
متجملانه
متحد
متحده
متحرک
متحمل
متحول
متخصص
متخلص
متدینی
متذکر
متروک
متروکه
متزلزل
متشکر
متشکل
متصل
متصور
متضاد
متضرر
متضمن
متعادل
متعارض
متعارف
متعال
متعدد
متعددی
متعصب
متعلق
متعهد
متغیر
متفاوت
متفاوتی
متفرق
متفکر
متقابل
متقاضی
متقن
متقی
متلاشی
متمادی
متمایل
متمرکز
متمم
متمولی
متناسب
متناقض
متنبی
متنبی‌خوان
متنوع
متنوعی
متهم
متواضع
متوالی
متوجه
متوسط
متوسطه
متوسل
متوفی
متوقف
متکثر
متکی
مثبت
مثبته
مثبتی
مجاز
مجانی
مجبور
مجدد
مجددی
مجذوب
مجرد
مجرم
مجروح
مجزا
مجسم
مجعول
مجلل
مجمل
مجهز
محافظتی
محافظه‌کار
محبوب
محتاج
محترم
محترمی
محتمل
محتوم
محدود
محدودکننده
محدودی
محروم
محرک
محسوب
محسوس
محض
محفوظ
محقق
محلی
محو
محوری
محکم
محکمی
محکم‌تری
محکوم
محیطی
مخابراتی
مخالف
مختصر
مختصری
مختل
مختلف
مختلفی
مخدر
مخدوش
مخرب
مخصوص
مخفی
مخلص
مخلوع
مد
مداخله‌جویانه
مدارا
مداوم
مدبرانه
مدرسی
مدرن
مدعی
مدنی
مدور
مدون
مدیدی
مدیریتی
مدیون
مذموم
مذهبی
مذهبی‌
مذهبی‌سنتی
مذکر
مذکور
مراقبتی
مربع
مربوط
مربوطه
مرتب
مرتبط
مرتفع
مرتکب
مردانه
مردد
مردمی
مردم‌نواز
مرده
مرزی
مرسوم
مرمت‌شده
مرمری
مرموز
مرهون
مرکب
مرکزی
مرگبار
مریض
مزبور
مزدور
مزمن
مزید
مسؤله‌داری
مسئله‌دار
مسئول
مسئولیت‌آور
مساعد
مسافرکش
مسافری
مسالمت‌آمیز
مساولانه
مساوی
مست
مستبدی
مستثنی
مستحضر
مستحق
مستحکم
مستحکمی
مستدل
مستعد
مستقر
مستقل
مستقلی
مستقیم
مستمر
مستندگونه
مستور
مستوی
مستی‌بخش
مسجل
مسدود
مسدود‌شده
مسرفانه
مسلح
مسلط
مسلم
مسلمان
مسلمانِ
مسموم
مسمومی
مسن
مسوول
مسکن
مسکونی
مسکوک
مسیحی
مشابه
مشابهی
مشارکتی
مشارکت‌جو
مشاهده‌شده
مشبک
مشتاق
مشتبه
مشترک
مشترکی
مشتمل
مشخص
مشخصی
مشرف
مشروط
مشروطه
مشروع
مشغول
مشفق
مشمول
مشهودی
مشهور
مشهوری
مشورتی
مشکل
مشکل‌گشا
مشکوک
مصدوم
مصرفی
مصرف‌شده
مصرف‌کننده
مصری
مصلحانه
مصمم
مصنوعی
مصوب
مصون
مضاعف
مضاف
مضر
مطابق
مطالعاتی
مطبوع
مطبوعاتی
مطرح
مطرح‌شده
مطروحه
مطرود
مطلع
مطلق
مطلقه
مطلوب
مطلوبی
مطمئن
مظلوم
معادل
معارض
معاصر
معاند
معتاد
معتبر
معتبری
معتدل
معترض
معتقد
معجزه‌آسایی
معدنی
معدود
معدودی
معذور
معرفتی
معرق
معروف
معزول
معصوم
معضل‌ساز
معطر
معطل
معطوف
معظم
معقول
معقولی
معلا
معلق
معلمی
معلوم
معماگونه
معمم
معممی
معمول
معمولی
معنایی
معنوی
معنی‌دار
معوقه
معکوس
معیشتی
مغبون
مغذی
مغرض
مغرور
مغزی
مغفول
مغلوب
مغناطیسی
مفت
مفرغی
مفصل
مفصلی
مفقود
مفقوده
مفقود‌شده
مفهومی
مفید
مقابل
مقاربتی
مقاوم
مقبول
مقتضی
مقدس
مقدم
مقدماتی
مقدور
مقرر
مقرون
مقصر
مقید
مقیم
ملایم
ملحد
ملزوم
ملموس
ملکوتی
ملکولی
ملکی
ملی
ملی‌گرای
ممتاز
ممتازی
ممزوج
ممنوع
ممنوعه
ممنون
ممکن
مناسب
مناسبی
مناسب‌تری
منافق
منان
منبت‌کاری‌شده
منتخب
منتسب
منتشر
منتشرشده
منتشره
منتشر‌شده
منتظر
منتفی
منتقل
منتهی
منثور
منجر
منحرف
منحصر
منحصربه‌فرد
منحصربه‌فردی
منحصر‌بفرد
منحصر‌به‌فرد
منحط
منحل
منحنی
مندرج
منزوی
منسجم
منسوخ
منصرف
منصفانه
منصفه
منصوب
منطبق
منطقی
منظم
منظوم
منعقد
منعکس
منفجره
منفور
منفک
منفک‌شده
منفی
منقوش
منقول
منهدم
منوط
منکر
مهربان
مهربانی
مهرشده
مهلک
مهم
مهمی
مهم‌
مهندسی
مه‌آلود
مواجه
مواجه‌
موازی
مواظب
موافق
موثق
موثقی
موجه
موجهی
موجود
مودب
موردی
موزون
موسوم
موضوعی
موظف
موفق
موفقی
موفقیت‌آمیز
موقت
موقوفه
مولد
مونت
موهوم
موکول
مکانیکی
مکتوبی
مکرر
مکرری
مکشوف
مکشوفه
مکلف
مکمل
میانجی
میانسالی
میانه‌رو
میانی
میان‌بر
میان‌سال
میان‌مدت
میخواره
میدانی
میزبان
میسر
میلادی
میکروبی
میگرنی
می‌نوش
ناآشنا
ناآگاه
نائل
نااقلیدسی
ناامن
ناامید
ناب
ناباب
نابود
ناتوان
ناتورالیستی
ناخالص
ناخرسند
ناخشنود
ناخواسته
ناخوشایند
نادر
نادرست
نادرستی
نادیده
ناراحت
ناراضی
ناراضی‌
نارس
ناروا
ناز
نازل
نازک
نازکی
ناشایست
ناشناخته
ناشناس
ناشی
ناظر
نافرجام
ناقص
نالان
نامالوف
نامتجانس
نامتعارف
نامحتمل
نامحدود
نامحدودی
نامدار
نامداری
نامربوط
نامریی
نامساعد
نامشخصی
نامطلوب
نامطمئن
نامناسب
نامنظم
نامنقح
ناموزون
ناموفق
نامی
نامیده
نام‌آشنای
نانوشته
نان‌شناس
ناهمزمان
ناپخته
ناپدید
ناپیدا
ناچار
ناچیز
ناچیزی
ناکارآمد
ناکافی
ناکام
ناگهانی
نایاب
نایب‌قهرمانی
نا‌ملموس
نباتی
نبوده
نبوغ‌آمیز
نترس
نجاتگر
نجات‌بخش
نجنگیده
نجومی
نجیب
نخست
نخودی
نرم
نرم‌افزاری
نزدیک
نزدیکی
نزولی
نساجی
نسبی
نستعلیق
نسطوری
نشانگر
نظارتی
نظامی
نظام‌یافته
نظری
نظیف
نظیفی
نفتی
نفره
نفوذی
نفی‌کننده
نقد
نقدی
نقلی
نقیض
نمادینی
نمایان
نمایانی
نمایشی
نمایشی‌
نمرده
نمناک
نموده
نمونه
نهاده
نهادی
نهادینه
نهان
نهایی
نهفته
نو
نوجوان
نورانی
نورشناختی
نوزدهم
نوشته
نوپای
نوینی
نویی
نپخته
نژادی
نگار
نگاشته
نگران
نگهدارنده
نگینی
نیازمند
نیاورده
نیاکانی
نیایشی
نیرومند
نیرومندی
نیشابوری
نیفزوده
نیل‌گون
نیمه‌تمام
نیمه‌شناخته‌شده
نیمه‌نهایی
نیمه‌پایانی
نیمه‌گرمسیری
نیم‌خیز
نیوزیلندی
نیک
نیکو
نیکوکار
هالیوودی
هانمید
هجدهم
هجری
هخامنشی
هدایتگری
هدایت‌کننده
هدفمند
هراسان
هرزه‌نویس
هرقلی
هر‌از‌چند‌گاه
هزار‌تومانی
هزینه‌شده
هشتم
هشت‌ساله
هشت‌هزار‌متری
هشداردهنده
هشدار‌دهنده
هفتم
هفتگی
هفدهم
هلاکوای
هلندی
هلکو
هماهنگ
همجوار
همدردی‌برانگیز
همراه
همرنگ
همزمان
همسان
همسایه
همسایهٔ
همسن
همصدا
همفکر
همه‌جانبه
همه‌جانبه‌یی
همه‌ساله
همه‌گیر
هموار
همپای
همپایهٔ
همگام
همگانی
همگروه
همگون
همیشگی
هم‌آهنگ‌کننده
هم‌جناح
هم‌خانواده
هم‌دندان
هم‌سان
هم‌سن
هم‌طیف
هم‌نظر
هم‌گروه
هند
هندشناس
هندی
هنری
هنگفت
هوادار
هوایی
هوشیار
هویدا
هیدروسفالی
هیچکاره
هیچکارهٔ
وابسته
وابستهٔ
وابسته‌
واحد
واحده
واحدی
وارد
وارداتی
وارده
وارد‌شده
وارفته
واسط
واسطه‌گری
واضح
واضحی
واضح‌تری
وافر
واقع
واقعی
واقع‌بینانه
واقع‌بینانه‌تری
واقع‌گرا
واقع‌گرای
واقف
واقفند
والا
وانمود
واهی
واگذار
واگذارشده
واگرایانه
واگیردار
وجودی
وحدت‌آمیز
وحشتناک
وحشت‌آور
وحشی
ورزشکار
ورزشی
ورودی
وسطی
وسیع
وسیعی
وصف‌ناشدنی
وصله‌زده
وضعی
وضعیتی
وضوح
وفادار
وقفی
ونزوالایی
وهم‌آلود
ویران
ویرانش
ویران‌کننده
ویروسی
ویژه
ویژه‌یی
پادشاهی
پارسی
پارلمانی
پاره
پاره‌پاره
پانزدهم
پاک
پاکدامن
پاکستانی
پاکش
پایانی
پایان‌بخش
پایان‌ناپذیری
پایبند
پایدار
پای‌بند
پخته
پخته‌شده
پخش‌شده
پدرانه
پدید
پذیرا
پذیرای
پذیرایی
پذیرفتنی
پذیرفته
پر
پرآشوب
پراحساس
پراندوهی
پراهمیتی
پراکنده
پربار
پرباغ
پربرف
پرتجملی
پرتنش
پرتیراژ
پرثمر
پرخاشگرانه
پرخروش
پرخواننده
پرداخته
پرداختی
پرداخت‌نشده
پردرآمد
پررنگ
پرزرق
پرسابقه
پرسابقهٔ
پرسود
پرسُخُن
پرشور
پرشکوه
پرشکوهی
پرطرفدار
پرمحتوا
پرمحصول
پرمخاطب
پرنده‌خوار
پرنشاط
پرنور
پرنکته
پرنگار
پرهیبتی
پرواضح
پرورده
پروسیله‌تری
پروپاقرص
پرپر
پرپینه
پرکار
پرگذشت
پرگل
پرگو
پر‌خطر
پر‌دردسری
پر‌رویا
پر‌قدرتی
پر‌مسئولیت
پر‌معنایی
پر‌گذشت
پزشکی
پستی
پسندیده
پشتگرم
پشت‌سبزی
پشیمان
پشیمون
پف
پلاستیکی
پلیسی
پنجاه‌ساله
پنجاه‌نفره
پنجاه‌نفری
پنجساله
پنجم
پنجمی
پنهان
پهن
پهناور
پهن‌شده
پوستی
پوشیده
پولدار
پولی
پوک
پُرنقش
پژوهشی
پیاده
پیاپی
پیدا
پیر
پیرامونی
پیروز
پیشتاز
پیشرفته
پیشرو
پیش‌آمده
پیش‌بینی‌شده
پیش‌پا‌افتاده
پیش‌کسوت
پیغمبری
پیمان‌شکن
پیوسته
پیچیده
پیگیر
چالاک
چاپاری
چاپی
چاپ‌شده
چرخان
چرخانده
چریکی
چسبیده
چشمگیر
چشمگیری
چشم‌انداز
چشم‌نواز
چمدانی
چنانی
چند
چندانی
چندبعدی
چندساله
چندگانه‌اندیش
چند‌
چند‌بعدی
چند‌جانبه
چند‌سالهٔ
چند‌هزار‌ساله
چنینی
چهاردهم
چهاردهمی
چهارساله
چهارم
چهارمی
چهارگانه
چهل‌نفره
چوبی
چپ
چیره
چینی
ژاپنی
ژرف‌نگر
ژنتیک
ژنتیکی
ژنریک
ژنی
کاذب
کارآمد
کاربردی
کارشناسانه
کارگری
کارگشا
کاری
کاسته
کافر
کافی
کامل
کاملی
کامور
کامکار
کانادایی
کانتینری
کاوش
کایب
کبیر
کتابی
کتانی
کتبی
کتمان‌ناپذیر
کثیر
کثیرالانتشار
کثیری
کثیف
کذب
کر
کربنیزه
کرد
کرمانی
کرم‌خورده
کرم‌خورده‌
کریم
کساد
کسروی
کسری
کشته
کشت‌شده
کشف‌شده
کشنده
کشوری
کشیده
کل
کلاسیک
کلافه
کلامی
کلان
کلان‌نگر
کلک
کلی
کلیدی
کلیشه
کلیوی
کم
کمتری
کمدی
کمرنگ
کمری
کمونیست
کمونیستی
کمی
کمیاب
کمیک
کم‌جمعیت
کم‌حرف
کم‌رنگ
کم‌سابقه
کم‌سن
کم‌ظرفیت
کم‌نظیر
کنجکاو
کنجکاوی
کند
کنده
کندی
کنونی
کهن
کهنسال
کهنه
کهنهٔ
کوتاه
کوتاهی
کوتاه‌مدت
کوته‌بینانه
کودن
کورکننده
کورکورانه
کوهنورد
کوچولویی
کوچک
کوچکی
کویتی
کویری
کیفی
کیلومتری
گازی
گازی‌شکل
گدا
گذشته
گذشته‌
گرامی
گران
گرانبار
گرانبها
گرانبهای
گرانقدر
گران‌بهایی
گردن‌کلفتِ
گردون
گرفتار
گرفته
گرفته‌شده
گرم
گرمسیری
گروهی
گروگانگیر
گریزان
گزارشگر
گزارشی
گزنده‌گو
گزینشی
گسترده
گسترده‌تری
گشاد‌کننده
گشوده
گفتاری
گفتنی
گفته
گلخانه‌مانند
گله‌مند
گلی
گل‌افشان
گم
گمراه
گمراه‌کننده
گمنام
گنبدی
گهربار
گوارای
گوارشی
گوشتی
گوناگون
گوناگونی
گویا
گویای
گیاهی
گیج
یائسه
یادآور
یادشده
یاد‌شده
یازدهم
یافت‌شده
یخی
یزدی
یمنی
یهودی
یونانی
یکجا
یکجای
یکسان
یکسانی
یکسویه
یکم
یکه
یکپارچه
یک‌جانبه
یک‌دست
یک‌روزه
ییلاقی
""".split('\n'))



'''Test Doc sets up tokens correctly.'''
from __future__ import unicode_literals
import pytest

from ...vocab import Vocab
from ...tokens.doc import Doc
from ...lemmatizer import Lemmatizer


@pytest.fixture
def lemmatizer():
    return Lemmatizer(lookup={'dogs': 'dog', 'boxen': 'box', 'mice': 'mouse'})


@pytest.fixture
def vocab(lemmatizer):
    return Vocab(lemmatizer=lemmatizer)


def test_empty_doc(vocab):
    doc = Doc(vocab)
    assert len(doc) == 0


def test_single_word(vocab):
    doc = Doc(vocab, words=['a'])
    assert doc.text == 'a '
    doc = Doc(vocab, words=['a'], spaces=[False])
    assert doc.text == 'a'


def test_lookup_lemmatization(vocab):
    doc = Doc(vocab, words=['dogs', 'dogses'])
    assert doc[0].text == 'dogs'
    assert doc[0].lemma_ == 'dog'
    assert doc[1].text == 'dogses'
    assert doc[1].lemma_ == 'dogses'



# coding: utf8
from __future__ import unicode_literals

from .doc import Doc
from ..symbols import HEAD, TAG, DEP, ENT_IOB, ENT_TYPE


def merge_ents(doc):
    """Helper: merge adjacent entities into single tokens; modifies the doc."""
    for ent in doc.ents:
        ent.merge(tag=ent.root.tag_, lemma=ent.text, ent_type=ent.label_)
    return doc


def format_POS(token, light, flat):
    """Helper: form the POS output for a token."""
    subtree = dict([
        ("word", token.text),
        ("lemma", token.lemma_),  # trigger
        ("NE", token.ent_type_),  # trigger
        ("POS_fine", token.tag_),
        ("POS_coarse", token.pos_),
        ("arc", token.dep_),
        ("modifiers", [])
    ])
    if light:
        subtree.pop("lemma")
        subtree.pop("NE")
    if flat:
        subtree.pop("arc")
        subtree.pop("modifiers")
    return subtree


def POS_tree(root, light=False, flat=False):
    """Helper: generate a POS tree for a root token. The doc must have
    `merge_ents(doc)` ran on it.
    """
    subtree = format_POS(root, light=light, flat=flat)
    for c in root.children:
        subtree["modifiers"].append(POS_tree(c))
    return subtree


def parse_tree(doc, light=False, flat=False):
    """Make a copy of the doc and construct a syntactic parse tree similar to
    displaCy. Generates the POS tree for all sentences in a doc.

    doc (Doc): The doc for parsing.
    RETURNS (dict): The parse tree.

    EXAMPLE:
        >>> doc = nlp('Bob brought Alice the pizza. Alice ate the pizza.')
        >>> trees = doc.print_tree()
        >>> trees[1]
        {'modifiers': [
            {'modifiers': [], 'NE': 'PERSON', 'word': 'Alice', 'arc': 'nsubj',
             'POS_coarse': 'PROPN', 'POS_fine': 'NNP', 'lemma': 'Alice'},
            {'modifiers': [
                {'modifiers': [], 'NE': '', 'word': 'the', 'arc': 'det',
                 'POS_coarse': 'DET', 'POS_fine': 'DT', 'lemma': 'the'}],
             'NE': '', 'word': 'pizza', 'arc': 'dobj', 'POS_coarse': 'NOUN',
             'POS_fine': 'NN', 'lemma': 'pizza'},
            {'modifiers': [], 'NE': '', 'word': '.', 'arc': 'punct',
             'POS_coarse': 'PUNCT', 'POS_fine': '.', 'lemma': '.'}],
            'NE': '', 'word': 'ate', 'arc': 'ROOT', 'POS_coarse': 'VERB',
            'POS_fine': 'VBD', 'lemma': 'eat'}
    """
    doc_clone = Doc(doc.vocab, words=[w.text for w in doc])
    doc_clone.from_array([HEAD, TAG, DEP, ENT_IOB, ENT_TYPE],
                         doc.to_array([HEAD, TAG, DEP, ENT_IOB, ENT_TYPE]))
    merge_ents(doc_clone)  # merge the entities into single tokens first
    return [POS_tree(sent.root, light=light, flat=flat)
            for sent in doc_clone.sents]



# encoding: utf8
from __future__ import unicode_literals, print_function

from ...language import Language
from ...attrs import LANG
from ...tokens import Doc, Token
from ...tokenizer import Tokenizer
from .tag_map import TAG_MAP

import re
from collections import namedtuple

ShortUnitWord = namedtuple('ShortUnitWord', ['surface', 'lemma', 'pos'])

# XXX Is this the right place for this?
Token.set_extension('mecab_tag', default=None)

def try_mecab_import():
    """Mecab is required for Japanese support, so check for it.

    It it's not available blow up and explain how to fix it."""
    try:
        import MeCab
        return MeCab
    except ImportError:
        raise ImportError("Japanese support requires MeCab: "
                          "https://github.com/SamuraiT/mecab-python3")

def resolve_pos(token):
    """If necessary, add a field to the POS tag for UD mapping.

    Under Universal Dependencies, sometimes the same Unidic POS tag can
    be mapped differently depending on the literal token or its context
    in the sentence. This function adds information to the POS tag to 
    resolve ambiguous mappings.
    """

    # NOTE: This is a first take. The rules here are crude approximations.
    # For many of these, full dependencies are needed to properly resolve
    # PoS mappings.

    if token.pos == '連体詞,*,*,*':
        if re.match('^[こそあど此其彼]の', token.surface):
            return token.pos + ',DET'
        if re.match('^[こそあど此其彼]', token.surface):
            return token.pos + ',PRON'
        else:
            return token.pos + ',ADJ'
    return token.pos

def detailed_tokens(tokenizer, text):
    """Format Mecab output into a nice data structure, based on Janome."""

    node = tokenizer.parseToNode(text)
    node = node.next # first node is beginning of sentence and empty, skip it
    words = []
    while node.posid != 0:
        surface = node.surface
        base = surface # a default value. Updated if available later.
        parts = node.feature.split(',')
        pos = ','.join(parts[0:4])

        if len(parts) > 7:
            # this information is only available for words in the tokenizer dictionary
            base = parts[7]

        words.append( ShortUnitWord(surface, base, pos) )
        node = node.next
    return words

class JapaneseTokenizer(object):
    def __init__(self, cls, nlp=None):
        self.vocab = nlp.vocab if nlp is not None else cls.create_vocab(nlp)

        MeCab = try_mecab_import()
        self.tokenizer = MeCab.Tagger()

    def __call__(self, text):
        dtokens = detailed_tokens(self.tokenizer, text)
        words = [x.surface for x in dtokens]
        doc = Doc(self.vocab, words=words, spaces=[False]*len(words))
        for token, dtoken in zip(doc, dtokens):
            token._.mecab_tag = dtoken.pos
            token.tag_ = resolve_pos(dtoken)
            token.lemma_ = dtoken.lemma
        return doc

    # add dummy methods for to_bytes, from_bytes, to_disk and from_disk to
    # allow serialization (see #1557)
    def to_bytes(self, **exclude):
        return b''

    def from_bytes(self, bytes_data, **exclude):
        return self

    def to_disk(self, path, **exclude):
        return None

    def from_disk(self, path, **exclude):
        return self

class JapaneseDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters[LANG] = lambda text: 'ja'
    tag_map = TAG_MAP

    @classmethod
    def create_tokenizer(cls, nlp=None):
        return JapaneseTokenizer(cls, nlp)

class Japanese(Language):
    lang = 'ja'
    Defaults = JapaneseDefaults
    Tokenizer = JapaneseTokenizer

    def make_doc(self, text):
        return self.tokenizer(text)

__all__ = ['Japanese']



# coding: utf8
from __future__ import unicode_literals

from ...attrs import LIKE_NUM

_num_words = ['බින්දුව', 'බිංදුව', 'එක', 'දෙක', 'තුන', 'හතර', 'පහ', 'හය', 'හත',
              'අට', 'නවය', 'නමය', 'දහය', 'එකොළහ', 'දොළහ', 'දහතුන', 'දහහතර',
              'දාහතර', 'පහළව', 'පහළොව', 'දහසය', 'දහහත', 'දාහත', 'දහඅට',
              'දහනවය', 'විස්ස', 'තිහ', 'හතළිහ', 'පනහ', 'හැට', 'හැත්තෑව', 'අසූව',
              'අනූව', 'සියය', 'දහස', 'දාහ', 'ලක්ෂය', 'මිලියනය', 'කෝටිය',
              'බිලියනය', 'ට්‍රිලියනය']

def like_num(text):
    text = text.replace(',', '').replace('.', '')
    if text.isdigit():
        return True
    if text.count('/') == 1:
        num, denom = text.split('/')
        if num.isdigit() and denom.isdigit():
            return True
    if text.lower() in _num_words:
        return True
    return False


LEX_ATTRS = {
    LIKE_NUM: like_num
}



# coding: utf-8
from __future__ import unicode_literals

from ..util import get_doc


def test_issue309(en_tokenizer):
    """Test Issue #309: SBD fails on empty string"""
    tokens = en_tokenizer(" ")
    doc = get_doc(tokens.vocab, [t.text for t in tokens], heads=[0], deps=['ROOT'])
    doc.is_parsed = True
    assert len(doc) == 1
    sents = list(doc.sents)
    assert len(sents) == 1
